------------------------------------------------------
-- Memdrv1. Memory driver 1. 
-- Description: Two modes: Read image and Start the 
-- image processing
-- University of Alicante. Depto. TIC
-- Author: Sergio Cuenca
-- Date: 13/7/99
-------------------------------------------------------
-------------------------------------------------------
----  DATAPATH
-------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity datapath is
    port (
        clock: in STD_LOGIC;
        reset: in STD_LOGIC;
        load: in STD_LOGIC;
        clk_en: in STD_LOGIC;
        d_in: in STD_LOGIC_VECTOR (7 downto 0);
        dat_rg: in STD_LOGIC;
        nibmux: in STD_LOGIC;
        q_out: out STD_LOGIC_VECTOR (14 downto 0);
        nib: out STD_LOGIC_VECTOR (3 downto 0)
    );
end datapath;

architecture datapath_arch of datapath is
signal cnt:std_logic_vector (14 downto 0);
signal d: std_logic_vector (7 downto 0);
begin

-- Address generator ----------------------------------------
ADDGEN: process(clock,reset)
begin
	-- reset asynchronous
		if reset='1' then
			cnt <= (others=>'0'); -- initial address bank 0
	-- address counter increments on rising edge of clock
		elsif (clock'event and clock='1') then
			if clk_en='1' then
				if load='1' then
					cnt<="100000000000000"; -- initial address bank 1
				else
					cnt <= cnt + 1;
				end if;
			end if;	
		end if;
end process;
q_out<=cnt;

-- Data register -------------------------------------------
DATAREG: process(dat_rg,reset)
begin
	if reset='1' then
		d <= (others=>'0');
	elsif (dat_rg'event and dat_rg='1') then
		d <= d_in;
	end if;
end process;

-- Multiplexor ----------------------------------------------
with nibmux select 
	nib<=	d(3 downto 0) when '0',
		d(7 downto 4) when others;
end datapath_arch;

-------------------------------------------------------
----  FSM
-------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

library SYNOPSYS;
use SYNOPSYS.attributes.all;

entity fsmdrv is 
  port (bnk: in STD_LOGIC;
        busy: in STD_LOGIC;
        clock: in STD_LOGIC;
        endproc: in STD_LOGIC;
        ini: in STD_LOGIC;
        reset: in STD_LOGIC;
        dat_rg: out STD_LOGIC;
        m_ctrl: out STD_LOGIC;
        m_oe: out STD_LOGIC;
        m_we: out STD_LOGIC;
        nibmux: out STD_LOGIC;
        r_en: out STD_LOGIC;
        r_ld: out STD_LOGIC;
        val: out STD_LOGIC);
end;

architecture fsmdrv_arch of fsmdrv is


-- SYMBOLIC ENCODED state machine: Sreg0
type Sreg0_type is (DTV1, DTV2, FIN, INC, INIT, LOAD, PROC, READ, RSTGB, WT);
signal Sreg0: Sreg0_type;

begin
--concurrent signal assignments
--diagram ACTIONS;


Sreg0_machine: process (clock, reset)

begin

if reset='1' then
	Sreg0 <= RSTGB;
elsif clock'event and clock = '1' then
	case Sreg0 is
		when DTV1 =>
			if busy='1' then
				Sreg0 <= WT;
			end if;
		when DTV2 =>
			if busy='1' then
				Sreg0 <= INC;
			end if;
		when FIN =>
		when INC =>
			Sreg0 <= READ;
		when INIT =>
			if ini='0' then
				Sreg0 <= PROC;
			elsif ini='1' then
				Sreg0 <= LOAD;
			end if;
		when LOAD =>
			Sreg0 <= DTV1;
		when PROC =>
			if endproc='1' then
				Sreg0 <= FIN;
			end if;
		when READ =>
			if busy='0' then
				Sreg0 <= LOAD;
			end if;
		when RSTGB =>
			if bnk='1' then
				Sreg0 <= INIT;
			elsif bnk='0' then
				Sreg0 <= INIT;
			end if;
		when WT =>
			if busy='0' then
				Sreg0 <= DTV2;
			end if;
		when others =>
			null;
	end case;
end if;
end process;

-- signal assignment statements for combinatorial outputs
r_en_assignment:
r_en <= '1' when (Sreg0 = INC) else
        '1' when (Sreg0 = RSTGB and bnk='1') else
        '0';

r_ld_assignment:
r_ld <= '1' when (Sreg0 = RSTGB and bnk='1') else
        '0';

nibmux_assignment:
nibmux <= '1' when (Sreg0 = DTV2) else
          '1' when (Sreg0 = INC) else
          '1' when (Sreg0 = READ) else
          '0';

val_assignment:
val <= '1' when (Sreg0 = DTV1) else
       '1' when (Sreg0 = DTV2) else
       '0';

dat_rg_assignment:
dat_rg <= '1' when (Sreg0 = LOAD) else
          '0';

m_oe_assignment:
m_oe <= '0';

m_we_assignment:
m_we <= '1';

m_ctrl_assignment:
m_ctrl <= '0' when (Sreg0 = PROC) else
          '1';

end fsmdrv_arch;
-------------------------------------------------------
----  TOP
-------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;

entity memdrv is
    port (
        reset: in STD_LOGIC;
        clock: in STD_LOGIC;
        ini: in STD_LOGIC;
        bnk: in STD_LOGIC;
        busy: in STD_LOGIC;
        endproc: in STD_LOGIC;
        d_in: in STD_LOGIC_VECTOR (7 downto 0);
        val: out STD_LOGIC;
        m_ctrl:out STD_LOGIC;
        m_oe:out STD_LOGIC;
        m_we:out STD_LOGIC;
        addr: out STD_LOGIC_VECTOR (14 downto 0);
        nib: out STD_LOGIC_VECTOR (3 downto 0)
    );
end memdrv;

architecture memdrv_arch of memdrv is

	component datapath
	port(clock: in STD_LOGIC;
        reset: in STD_LOGIC;
        load: in STD_LOGIC;
        clk_en: in STD_LOGIC;
        d_in: in STD_LOGIC_VECTOR (7 downto 0);
        dat_rg: in STD_LOGIC;
        nibmux: in STD_LOGIC;
        q_out: out STD_LOGIC_VECTOR (14 downto 0);
        nib: out STD_LOGIC_VECTOR (3 downto 0));
    	end component;
    	
    	component fsmdrv
    	port (bnk: in STD_LOGIC;
        busy: in STD_LOGIC;
        clock: in STD_LOGIC;
        endproc: in STD_LOGIC;
        ini: in STD_LOGIC;
        reset: in STD_LOGIC;
        dat_rg: out STD_LOGIC;
        m_ctrl: out STD_LOGIC;
        m_oe: out STD_LOGIC;
        m_we: out STD_LOGIC;
        nibmux: out STD_LOGIC;
        r_en: out STD_LOGIC;
        r_ld: out STD_LOGIC;
        val: out STD_LOGIC);
        end component;

        
signal datrg,mux,ren,rld: std_logic;

begin
u0: datapath port map(clock=>clock, reset=>reset, load=>rld, clk_en=>ren,
		d_in=>d_in, dat_rg=>datrg, nibmux=>mux, q_out=>addr,nib=>nib);

u1: fsmdrv port map(bnk=>bnk, busy=>busy, clock=>clock, endproc=>endproc, ini=>ini,reset=>reset,
		dat_rg=>datrg, m_ctrl=>m_ctrl, m_oe=>m_oe, m_we=>m_we, nibmux=>mux, r_en=>ren, r_ld=>rld,
		val=>val);		

end memdrv_arch;
